@@ -1,9 +1,11 @@
+use crate::utils::{graph as util_graph, ip_blacklist};
 use chrono::Local;
+use chrono::{DateTime, Datelike, FixedOffset};
 use csv::{Reader, Writer};
 use reqwest;
 use rustc_serialize::json::Json;
 use serde::{Deserialize, Serialize};
-use std::{error::Error, fs::File, path::Path};
+use std::{collections::HashMap, error::Error, fs::File, path::Path};
 use termion::color;
 use whois::WhoIs;
 
@@ -157,7 +159,14 @@ impl<'a> Tracker<'a> {
         Ok(())
     }
 
-    pub fn history(&self, id: &str) -> Result<(), Box<dyn Error>> {
+    pub fn history(
+        &self,
+        id: &str,
+        oneline: bool,
+        all: bool,
+        graph: bool,
+    ) -> Result<(), Box<dyn Error>> {
+        let mut show_his = History::new(oneline, all);
         let _secret = self.get_secret(id.to_string());
         if let Ok(secret) = _secret {
             let url = format!("{}/u?id={}&key={}", self.api, id, secret);
@@ -165,28 +174,10 @@ impl<'a> Tracker<'a> {
             let res = reqwest::blocking::get(url)?;
             let _result = res.json::<Vec<HistoryRes>>();
             if let Ok(result) = _result {
-                for element in result {
-                    println!(
-                        "💿 {}{}{}",
-                        color::Fg(color::Magenta),
-                        element.unique_id,
-                        color::Fg(color::Reset)
-                    );
-                    println!(
-                        "\t💡 IP address: {}{}{}",
-                        color::Fg(color::LightGreen),
-                        element.ip,
-                        color::Fg(color::Reset)
-                    );
-                    println!(
-                        "\t📆 Date: {}{}{}",
-                        color::Fg(color::LightGreen),
-                        element.time,
-                        color::Fg(color::Reset)
-                    );
-                    self.whois(element.ip)?;
-
-                    println!("");
+                if graph {
+                    show_his.print_graph(result)?;
+                } else {
+                    show_his.print(result)?;
                 }
             } else {
                 println!("empty history.")
@@ -259,13 +250,76 @@ impl<'a> Tracker<'a> {
         wtr.flush()?;
         Ok(())
     }
+}
 
-    fn whois(&self, ip: String) -> Result<(), Box<dyn Error>> {
+struct History {
+    cache: HashMap<String, String>,
+    oneline: bool,
+    all: bool,
+}
+
+impl History {
+    pub fn new(oneline: bool, all: bool) -> Self {
+        Self {
+            cache: HashMap::new(),
+            oneline: oneline,
+            all: all,
+        }
+    }
+
+    pub fn print(&mut self, result: Vec<HistoryRes>) -> Result<(), Box<dyn Error>> {
+        for element in result {
+            if self.all || self.select_ip(&element.ip) {
+                match self.oneline {
+                    true => self.history_oneline(&element),
+                    false => self.history_multiline(&element)?,
+                }
+            }
+        }
+        Ok(())
+    }
+
+    pub fn print_graph(&mut self, result: Vec<HistoryRes>) -> Result<(), Box<dyn Error>> {
+        let mut plot_data: Vec<(DateTime<FixedOffset>, usize)> = vec![];
+
+        for element in result {
+            if self.all || self.select_ip(&element.ip) {
+                let date = DateTime::parse_from_rfc3339(&element.time)?;
+                let index = plot_data
+                    .iter()
+                    .position(|x| x.0.month() == date.month() && x.0.day() == date.day());
+
+                if let Some(index) = index {
+                    plot_data[index] = (plot_data[index].0, plot_data[index].1 + 1);
+                } else {
+                    plot_data.push((date, 1));
+                }
+            }
+        }
+
+        plot_data.sort_by(|x, y| y.0.cmp(&x.0));
+
+        util_graph::plot_barchart(plot_data)?;
+
+        Ok(())
+    }
+
+    fn whois(&mut self, ip: String) -> Result<(), Box<dyn Error>> {
         println!("\t👤 Whois:");
 
-        let mut whois = WhoIs::new(ip);
+        let lookup: String = match self.cache.get(&ip) {
+            Some(value) => value.clone(),
+            None => {
+                let mut whois = WhoIs::new(ip.clone());
+                let lookup = whois.lookup()?;
 
-        let json = &Json::from_str(&whois.lookup()?)?;
+                self.cache.insert(ip.clone(), lookup.clone());
+
+                lookup
+            }
+        };
+
+        let json = &Json::from_str(&lookup)?;
 
         if let Some(json_object) = json.as_object() {
             for (key, value) in json_object {
@@ -285,4 +339,48 @@ impl<'a> Tracker<'a> {
 
         Ok(())
     }
+
+    fn select_ip(&self, ip: &str) -> bool {
+        let result = ip_blacklist::IP_BLACKLIST.iter().position(|&r| r == ip);
+
+        result.is_none()
+    }
+
+    fn history_multiline(&mut self, history: &HistoryRes) -> Result<(), Box<dyn Error>> {
+        println!(
+            "💿 {}{}{}",
+            color::Fg(color::Magenta),
+            history.unique_id,
+            color::Fg(color::Reset)
+        );
+        println!(
+            "\t💡 IP address: {}{}{}",
+            color::Fg(color::LightGreen),
+            history.ip,
+            color::Fg(color::Reset)
+        );
+        println!(
+            "\t📆 Date: {}{}{}",
+            color::Fg(color::LightGreen),
+            history.time,
+            color::Fg(color::Reset)
+        );
+        self.whois(history.ip.clone())?;
+
+        println!("");
+
+        Ok(())
+    }
+
+    fn history_oneline(&self, history: &HistoryRes) {
+        println!(
+            "💿 {}{}{} - {}{}{}",
+            color::Fg(color::Magenta),
+            history.ip,
+            color::Fg(color::Reset),
+            color::Fg(color::LightGreen),
+            history.time,
+            color::Fg(color::Reset)
+        );
+    }
 }
